local function SpawnMagicSparkles(mo, sprkcolor, sparkx, sparky, sparkz, overlay, random)
	local spark = P_SpawnMobj(sparkx, sparky, sparkz, MT_THOK)
	if overlay
		spark.type = MT_OVERLAY
		spark.target = mo
		spark.spriteyoffset = P_RandomRange(-1, mo.height/FRACUNIT)*FRACUNIT
		spark.spritexoffset = P_RandomRange(-mo.radius/FRACUNIT, mo.radius/FRACUNIT)*FRACUNIT
	end
	if random
		local randomx = mo.x+FixedMul(P_RandomRange(-25,25)*FRACUNIT,sin(P_RandomRange(0,360)*ANG1))
		local randomy = mo.y+FixedMul(P_RandomRange(-25,25)*FRACUNIT,cos(P_RandomRange(0,360)*ANG1))
		local randomz = mo.z + P_RandomRange(-1, mo.height/FRACUNIT)*FRACUNIT
		P_TeleportMove(spark, randomx, randomy, randomz)
	end
	spark.state = S_MAGICSPARK1
	spark.color = sprkcolor
	if mo.eflags & MFE_VERTICALFLIP
		spark.eflags = $|MFE_VERTICALFLIP
		spark.flags2 = $|MF2_OBJECTFLIP
	else
		spark.eflags = $ & ~MFE_VERTICALFLIP
		spark.flags2 = $ & ~MF2_OBJECTFLIP
	end
	return spark
end

local MAGIC_NONE = 0
local MAGIC_FIRE = 1
local MAGIC_ICE = 2
local MAGIC_WATER = 3
local MAGIC_WIND = 4
local MAGIC_BOMB = 5
local MAGIC_MAGNET = 6
local MAGIC_ELEC = 7

local magiccolors = {SKINCOLOR_GREEN,SKINCOLOR_RED,SKINCOLOR_CYAN,SKINCOLOR_BLUE,SKINCOLOR_WHITE,SKINCOLOR_ORANGE,SKINCOLOR_YELLOW,SKINCOLOR_VAPOR}
local nonecolors = {SKINCOLOR_GREEN,SKINCOLOR_FOREST,SKINCOLOR_EMERALD,SKINCOLOR_SEAFOAM,SKINCOLOR_APPLE}
local firecolors = {SKINCOLOR_SALMON,SKINCOLOR_RED,SKINCOLOR_RUBY,SKINCOLOR_KETCHUP,SKINCOLOR_CRIMSON,SKINCOLOR_FLAME}
local icecolors = {SKINCOLOR_AZURE,SKINCOLOR_SKY,SKINCOLOR_SLATE,SKINCOLOR_BLUEBELL,SKINCOLOR_TEAL,SKINCOLOR_CYAN}
local watercolors = {SKINCOLOR_BLUE,SKINCOLOR_COBALT,SKINCOLOR_ICY,SKINCOLOR_CERULEAN,SKINCOLOR_CORNFLOWER,SKINCOLOR_AQUA}
local windcolors = {SKINCOLOR_WHITE,SKINCOLOR_AETHER,SKINCOLOR_GREY,SKINCOLOR_CARBON,SKINCOLOR_SILVER,SKINCOLOR_SLATE}
local bombcolors = {SKINCOLOR_SANDY,SKINCOLOR_ORANGE,SKINCOLOR_COPPER,SKINCOLOR_RUST,SKINCOLOR_SUNSET,SKINCOLOR_APRICOT}
local magnetcolors = {SKINCOLOR_YELLOW,SKINCOLOR_GOLD,SKINCOLOR_OLIVE,SKINCOLOR_PERIDOT,SKINCOLOR_APPLE,SKINCOLOR_LIME}
local eleccolors = {SKINCOLOR_ROSY,SKINCOLOR_PURPLE,SKINCOLOR_MAGENTA,SKINCOLOR_VAPOR,SKINCOLOR_NEON,SKINCOLOR_VIOLET}

local msshield = {
[0] = S_MSSHIELD_F1,
[1] = S_MSSHIELD_F2}

local bubblesounds = {
sfx_bubbl1, sfx_bubbl2,
sfx_bubbl3, sfx_bubbl4,
sfx_bubbl5}

local function SpawnMagicPentogram(mo, magicolor, addx, addy, addz, splat, magicfuse, mrollangle)
	local pent = P_SpawnMobjFromMobj(mo, addx, addy, addz, MT_THOK)
	pent.fuse = magicfuse
	pent.scale = 0
	pent.target = mo
	pent.destscale = FRACUNIT
	pent.state = S_MAGICPENTOGRAM1
	pent.color = magicolor
	pent.rollangle = mrollangle
	pent.tics = magicfuse
	
	local overlay = P_SpawnMobjFromMobj(mo, addx, addy, addz, MT_OVERLAY)
	overlay.fuse = magicfuse
	overlay.state = S_MAGICPENTOGRAM2
	overlay.color = magicolor
	overlay.target = pent
	overlay.scale = 0
	overlay.destscale = FRACUNIT
	overlay.rollangle = -mrollangle
	overlay.tics = magicfuse
	if splat
		pent.flags2 = $|MF2_SPLAT
		pent.renderflags = $|RF_FLOORSPRITE
		pent.angle = pent.rollangle
		pent.type = MT_OVERLAY
		pent.target = mo
		
		if overlay
			overlay.flags2 = $|MF2_SPLAT
			overlay.renderflags = $|RF_FLOORSPRITE
			overlay.angle = overlay.rollangle
		end	
	else
		pent.angle = mo.angle+ANGLE_90
		pent.spriteyoffset = addz
		pent.type = MT_THOK
		if overlay
			overlay.angle = mo.angle
			overlay.spriteyoffset = addz
		end
	end
end

addHook("ShieldSpecial", function(player)
	if player.valid and player.playerstate == PST_LIVE
	and player.mo.skin == "magicmetal" and not (player.powers[pw_carry] == CR_NIGHTSMODE)
		return true
	end
end)

addHook("PlayerThink", function(player)
	if player.valid and player.playerstate == PST_LIVE
	and player.mo.skin == "magicmetal" and not (player.powers[pw_carry] == CR_NIGHTSMODE)
		local target = P_LookForEnemies(player, true, true)
		if player.specialuses == nil
			player.specialuses = 10
		end
		if player.magic == MAGIC_NONE
			if player.specialuses < 10
				if (leveltime%50) == 1
					player.specialuses = $ + 1
				end
			end
		end
		if player.magicrollangle == nil
			player.magicrollangle = 0
		end
		player.magicrollangle = $ + 5*ANG1
		if player.halfuse == nil
			player.halfuse = 0
		end
		if player.magic == nil
			player.magic = 0
		end
		if player.magic > 7
			player.magic = 7
		end
		if player.magic < 0
			player.magic = 0
		end
		if player.mo.color != SKINCOLOR_MAGICALMETAL
			player.mo.blendmode = AST_ADD
		else
			player.mo.blendmode = 0
		end
		if player.cmd.buttons & BT_WEAPONNEXT
			if player.wepnext == false
				if player.magic < 7
					player.magic = $ + 1
					S_StartSound(player.mo, sfx_wepchg, player)
				end
			end
			player.wepnext = true
		else
			player.wepnext = false
		end
		if player.cmd.buttons & BT_WEAPONPREV
			if player.wepprev == false
				if player.magic > 0
					player.magic = $ - 1
					S_StartSound(player.mo, sfx_wepchg, player)
				end
			end
			player.wepprev = true
		else
			player.wepprev = false
		end
		if player.specialuses > 10
			player.specialuses = 10
		end
		if player.speeny
			local gotox = FixedMul(50*FRACUNIT, sin(player.drawangle+ANGLE_90))
			local gotoy = FixedMul(50*FRACUNIT, cos(player.drawangle+ANGLE_90))
			
			player.speeny = $ - 1
			player.mo.state = S_PLAY_CLIMB

			local fireball = P_SpawnMobjFromMobj(player.mo, 0, 0, 0, MT_CORK)
			if fireball
				//P_TeleportMove(fireball, player.mo.x, player.mo.y, player.mo.z)
				fireball.color = firecolors[P_RandomRange(1, 6)]
				fireball.colorized = true
				fireball.fuse = 2
				fireball.state = S_FIREBALLTRAIL1
				fireball.tracer = player.mo
				fireball.scale = FRACUNIT*2
				fireball.frame = $|FF_TRANS60
				fireball.blendmode = AST_ADD
			end
			
			local fire = P_LookForEnemies(player, false, true)
			
			for i = 1, 5
				local smolones = P_SPMAngle(player.mo, MT_FIREBALL, P_RandomRange(0, 360)*ANG1)
				if smolones
					if fire
						P_HomingAttack(smolones, fire)
					end
					smolones.momz = P_RandomRange(-10, 10)*FRACUNIT
					P_InstaThrust(smolones, P_RandomRange(0, 360)*ANG1, P_RandomRange(0, 30)*FRACUNIT)
					smolones.fuse = 15
					smolones.scale = FRACUNIT/P_RandomRange(1, 15)
				end	
			end	
			
			if P_IsObjectOnGround(player.mo)
				player.speeny = false
				if player.mo.state == S_PLAY_CLIMB
					player.mo.state = S_PLAY_STND
				end
			end
			if player.speeny == 1
				player.speeny = 0
				player.mo.state = S_PLAY_SPRING
				player.pflags = $|PF_NOJUMPDAMAGE|PF_JUMPED
				local ghost = P_SpawnGhostMobj(player.mo)
				ghost.destscale = player.mo.scale + FRACUNIT/2
				ghost.colorized = true
				ghost.color = firecolors[P_RandomRange(1, 6)]
			end
		end
		
		if player.magic == MAGIC_FIRE then SpawnMagicSparkles(player.mo, firecolors[P_RandomRange(1, 6)], 0, 0, 0, false, true) end
		if player.magic == MAGIC_ICE then SpawnMagicSparkles(player.mo, icecolors[P_RandomRange(1, 6)], 0, 0, 0, false, true) end
		if player.magic == MAGIC_WATER then SpawnMagicSparkles(player.mo, watercolors[P_RandomRange(1, 6)], 0, 0, 0, false, true) end
		if player.magic == MAGIC_WIND then SpawnMagicSparkles(player.mo, windcolors[P_RandomRange(1, 6)], 0, 0, 0, false, true) end
		if player.magic == MAGIC_BOMB then SpawnMagicSparkles(player.mo, bombcolors[P_RandomRange(1, 6)], 0, 0, 0, false, true) end
		//if player.magic == MAGIC_MAGNET then SpawnMagicSparkles(player.mo, magnetcolors[P_RandomRange(1, 6)], 0, 0, 0, false, true) end
		if player.magic == MAGIC_ELEC then SpawnMagicSparkles(player.mo, eleccolors[P_RandomRange(1, 6)], 0, 0, 0, false, true) end
		
		if not (player.name == "BuggieTheBug")
			if (player.magic == MAGIC_FIRE and emeralds < 1) then player.magic = 0 end
			if (player.magic == MAGIC_ICE and emeralds < 2) then player.magic = 0 end
			if (player.magic == MAGIC_WATER and emeralds < 4) then player.magic = 0 end
			if (player.magic == MAGIC_WIND and emeralds < 8) then player.magic = 0 end
			if (player.magic == MAGIC_BOMB and emeralds < 16) then player.magic = 0 end
			if (player.magic == MAGIC_MAGNET and emeralds < 32) then player.magic = 0 end
			if (player.magic == MAGIC_ELEC and emeralds < 64) then player.magic = 0 end
		end
		
		if player.followmobj and player.speed
		and not (player.mo.state == S_PLAY_ROLL or player.mo.state == S_PLAY_JUMP)
			player.followmobj.flags2 = $|MF2_DONTDRAW
			player.followmobj.state = S_MAGICSPARK3
			player.followmobj.spriteyscale = FRACUNIT*5
			player.followmobj.spritexscale = FRACUNIT*5
			player.followmobj.scale = FRACUNIT/2
			
			if (leveltime&4) == 0
				local sparks = SpawnMagicSparkles(player.followmobj, P_RandomRange(1, #skincolors-1), 0, 0, 0, false, true)
				sparks.momx = P_RandomRange(-5, 5)*FRACUNIT
				sparks.momy = P_RandomRange(-5, 5)*FRACUNIT
				sparks.momz = P_RandomRange(-2, 1)*FRACUNIT
			end	
		end
		if player.cd
			player.cd = $ - 1
		end
		if player.metaljump
			player.metaljump = $ - 1
			P_SetObjectMomZ(player.mo, FRACUNIT, true)
			if (leveltime%5) == 0
				local ghost = P_SpawnGhostMobj(player.mo)
				ghost.destscale = 0
				ghost.color = SKINCOLOR_SKY
				ghost.colorized = true
			end	
			if player.mo.momz == 0
			and player.metaljump > 2
				P_SetObjectMomZ(player.mo, 5*FRACUNIT)
			end
			if player.metaljump == 1
				player.mo.momz = $/2
			end
			if player.metaljump < 20
				player.mo.state = S_PLAY_SPRING
			end
		end
		if not player.cd
			if (player.cmd.buttons & BT_WEAPONMASK)
				if not player.weapon
					local weapon = (player.cmd.buttons & BT_WEAPONMASK)
					if weapon != player.magic
						player.magic = weapon
					else
						player.magic = MAGIC_NONE
					end	
					player.cd = 3
					player.menu = false
					player.jumpfactor = skins[player.mo.skin].jumpfactor
				end	
				player.weapon = true
			else
				player.weapon = false
			end
		end
		if player.magic == MAGIC_WATER
		and not (player.powers[pw_shield] & SH_PROTECTWATER)
			if player.specialuses
			and player.mo.eflags & MFE_UNDERWATER
				if player.specialuses > 1
					player.powers[pw_underwater] = 30*TICRATE
					SpawnMagicPentogram(player.mo, watercolors[P_RandomRange(1, 6)], 0, 0, FRACUNIT/15, true, 2, player.magicrollangle)
				end
				if (leveltime%50) == 0
					player.specialuses = $ - 1
				end
			end	
		end
		if player.bubblebounce
			player.bubblebounce = $ - 1
			player.mo.state = S_PLAY_CLIMB
			player.mo.flags = $|MF_BOUNCE
			if P_IsObjectOnGround(player.mo)
				P_DoBubbleBounce(player)
				player.mo.state = S_PLAY_STND
			end	
			if (leveltime%5) == 0
				local ghost = P_SpawnGhostMobj(player.mo)
				ghost.destscale = 0
				ghost.color = watercolors[P_RandomRange(1, 6)]
				ghost.colorized = true
				S_StartSound(player.mo, bubblesounds[P_RandomRange(1, 5)])
			end	
			if player.bubblebounce == 1
				player.bubblebounce = 0
				local ghost = P_SpawnGhostMobj(player.mo)
				ghost.destscale = FRACUNIT*2
				ghost.color = watercolors[P_RandomRange(1, 6)]
				ghost.colorized = true
				ghost.state = S_PLAY_SPRING
				player.mo.state = S_PLAY_SPRING
				S_StartSound(player.mo, sfx_bnce2)
				player.mo.flags = $ & ~MF_BOUNCE
			end
		end
		/*if player.flying
			if player.cmd.buttons & BT_JUMP
			and player.pflags & PF_THOKKED
				if player.jump == false
					local ghost = P_SpawnGhostMobj(player.mo)
					ghost.destscale = FRACUNIT*2
					ghost.color = windcolors[P_RandomRange(1, 6)]
					ghost.colorized = true
				end
				if player.jump == true
					local ghost = P_SpawnGhostMobj(player.mo)
					ghost.destscale = 0
					ghost.color = SKINCOLOR_AETHER
					ghost.colorized = true
					ghost.fuse = 2
					P_SetObjectMomZ(player.mo, FRACUNIT, true)
					player.mo.state = S_PLAY_SPRING
					player.drawangle = (30*player.realtime)*ANG1
					
					for i = 1, 4
						local ns = player.drawangle + (ANGLE_90*i)
						
						local windx = FixedMul(50*FRACUNIT, sin(ns))
						local windy = FixedMul(50*FRACUNIT, sin(ns))
						
						local wind = P_SpawnMobjFromMobj(player.mo, windx, windy, player.mo.height/2, MT_DUSTLAYER)
						wind.frame = $|FF_PAPERSPRITE
						wind.fuse = 5
						wind.scale = FRACUNIT/2
						wind.angle = ns+ANGLE_90
						wind.colorized = true
						wind.color = windcolors[P_RandomRange(1, 6)]
					end	
					player.flying = $ - 1
				end
			else
				if player.jump == true
					local ghost = P_SpawnGhostMobj(player.mo)
					ghost.destscale = FRACUNIT*2
					ghost.color = windcolors[P_RandomRange(1, 6)]
					ghost.colorized = true
				end
			end
		end*/
		if player.float
			if player.cmd.buttons & BT_JUMP
			and player.pflags & PF_THOKKED
				player.float = $ - 1
				player.mo.momz = 0
				player.mo.state = S_PLAY_FLY
				SpawnMagicSparkles(player.mo, eleccolors[P_RandomRange(1, 6)], 0, 0, 0, false, true)
			end
			if player.float == 1
				local ghost = P_SpawnGhostMobj(player.mo)
				ghost.destscale = player.mo.scale + FRACUNIT/2
				ghost.colorized = true
				ghost.color = magnetcolors[P_RandomRange(1, 6)]
				player.mo.state = S_PLAY_FALL
				S_StartSound(player.mo, sfx_appear)
			end
			if P_IsObjectOnGround(player.mo)
				player.float = 0
				if player.mo.state == S_PLAY_FLY
					player.mo.state = S_PLAY_STND
				end
			end
		end
		/*if player.iceroad
			player.iceroad = $ - 1
			player.mo.momz = 0
			player.mo.state = S_PLAY_RUN
			P_InstaThrust(player.mo, player.mo.angle, 30*FRACUNIT)
			player.drawangle = player.mo.angle
			player.cmd.sidemove = 0
			local ice = P_SpawnMobjFromMobj(player.mo, 0, 0, -2*FRACUNIT, MT_THOK)
			ice.flags2 = $|MF2_SPLAT
			ice.renderflags = $|RF_NOSPLATBILLBOARD
			ice.flags = $|MF_SOLID|MF_NOGRAVITY
			ice.height = FRACUNIT
			ice.blendmode = AST_ADD
			ice.color = icecolors[P_RandomRange(1, 6)]
			ice.colorized = true
			ice.fuse = 15
			ice.state = S_FHZICE1
			ice.angle = player.mo.angle+ANGLE_180
			if player.cmd.buttons & BT_JUMP
			and player.jump == false
				player.iceroad = 0
				P_SetObjectMomZ(player.mo, 8*FRACUNIT)
				P_DoJump(player)
				player.pflags = $|PF_THOKKED
				S_StartSound(player.mo, sfx_shattr)
			end
		end*/
		if player.stap
			if (leveltime%1) == 0
				player.stap = $ - 1
			end	
			player.mo.colorized = true
			player.mo.state = S_PLAY_CLIMB
			if player.stap <= 20
				P_SetObjectMomZ(player.mo, -30*FRACUNIT)
				player.mo.momx = 0
				player.mo.momy = 0
			else
				player.mo.momz = 0
				player.mo.color = SKINCOLOR_ICY
			end	
			if not P_IsObjectOnGround(player.mo)
				player.pflags = $|PF_FULLSTASIS
				player.powers[pw_invulnerability] = 2
			else
				if not player.grd
					S_StartSound(player.mo, sfx_thwomp)
					for i = 1, 36
						local ns = i*(10*ANG1)
						local shock = P_SPMAngle(player.mo, MT_DEMONFIRE, ns)
						P_InstaThrust(shock, ns, 15*FRACUNIT)
						shock.fuse = 15
						shock.state = S_SNOW2
						shock.scale = FRACUNIT*2
						shock.tracer = player.mo
						
						local dust = P_SPMAngle(player.mo, MT_DUST, ns)
						P_InstaThrust(dust, ns, 15*FRACUNIT)
						dust.fuse = 50
						dust.blendmode = AST_ADD
						dust.colorized = true
						dust.scale = FRACUNIT*P_RandomRange(1, 3)
						dust.color = SKINCOLOR_WHITE
						dust.tracer = player.mo
					end
				end
				player.powers[pw_flashing] = 2
				player.pflags = $|PF_STASIS
				player.pflags = $ & ~PF_FULLSTASIS
				if player.cmd.buttons & BT_JUMP
					player.stap = 0
					player.mo.colorized = false
					P_DoJump(player)
					P_SetObjectMomZ(player.mo, 5*FRACUNIT)
					player.mo.state = S_PLAY_JUMP
					player.mo.color = player.skincolor
					player.pflags = $ & ~PF_STASIS
					S_StartSound(player.mo, sfx_shattr)
				end
			end
			if P_IsObjectOnGround(player.mo)
				player.grd = true
			else
				player.grd = false
			end
			if player.stap == 1
				player.stap = 0
				player.mo.colorized = false
				P_DoJump(player)
				P_SetObjectMomZ(player.mo, 5*FRACUNIT)
				player.mo.state = S_PLAY_JUMP
				player.mo.color = player.skincolor
				player.pflags = $ & ~PF_STASIS
				S_StartSound(player.mo, sfx_shattr)
			end
		end
		if player.zoomers
			player.zoomers = $ - 1
			local zoom = P_SpawnMobjFromMobj(player.mo, 0, 0, 0, MT_THOK)
			//zoom.state = S_ZOOM
			zoom.sprite = SPR_ZOOM
			zoom.color = player.mo.color
			zoom.frame = $|FF_PAPERSPRITE
			zoom.spritexscale = FRACUNIT*3 + FRACUNIT/5
			zoom.spriteyscale = FRACUNIT - FRACUNIT/3
			zoom.angle = R_PointToAngle2(0, 0, player.mo.momx, player.mo.momy)
			P_InstaThrust(player.mo, R_PointToAngle2(0, 0, player.mo.momx, player.mo.momy), 60*FRACUNIT)
			player.mo.blendmode = AST_ADD
			player.mo.momz = 0
			player.pflags = $|PF_SPINNING
			if player.zoomers == 1
				player.mo.momx = $/4
				player.mo.momy = $/4
				player.mo.momz = 0
				player.zoomers = 0
				player.mo.blendmode = 0
				player.pflags = $ & ~PF_SPINNING
				S_StartSound(player.mo, sfx_skid)
			end
		end
		if player.magic == MAGIC_MAGNET
			local addx = FixedMul(25*FRACUNIT, cos(player.mo.angle))
			local addy = FixedMul(25*FRACUNIT, sin(player.mo.angle))
		
			player.magnet = P_SpawnMobjFromMobj(player.mo, addx, addy, player.mo.height/2, MT_MAGNET)
			if player.magnet
				player.magnet.fuse = 2
				player.magnet.scale = FRACUNIT*2
				player.magnet.tracer = player.mo
				if target
					local angle = R_PointToAngle2(player.magnet.x, player.magnet.y, target.x, target.y)
					local dist = R_PointToDist2(player.magnet.x, player.magnet.y, target.x, target.y)
					player.magnet.angle = angle
					//SpawnMagicPentogram(player.mo, magnetcolors[P_RandomRange(1, 6)], 0, 0, FRACUNIT/15, true, 2, player.magicrollangle)
				else
					player.magnet.angle = player.mo.angle
					if not player.float
						SpawnMagicSparkles(player.mo, magnetcolors[P_RandomRange(1, 6)], 0, 0, 0, true)
					end
				end
			end
		end
		if player.cmd.buttons & BT_SPIN
			if not player.spin
			and not player.cd
			and player.specialuses
				player.cd = 15
				if player.magic != MAGIC_NONE
					player.specialuses = $ - 1
				end	
				if player.magic == MAGIC_NONE
					local energyball = P_SPMAngle(player.mo, MT_JETTBULLET, player.mo.angle)
					if energyball
						energyball.color = nonecolors[P_RandomRange(1, 5)]
						energyball.colorized = true
						energyball.flags = $|MF_NOGRAVITY
						energyball.state = S_JETFUME1
						energyball.fuse = 25
						energyball.tracer = player.mo
						player.pflags = $|PF_JUMPED|PF_NOJUMPDAMAGE
						player.cd = 25
						P_InstaThrust(energyball, player.mo.angle, 10*FRACUNIT)
						P_SetObjectMomZ(energyball, 0)
						S_StopSound(energyball)
						S_StartSound(player.mo, sfx_cybdth)
					end	
					P_SetObjectMomZ(player.mo, FRACUNIT*2, false)
					player.mo.state = S_PLAY_SPRING
				end
				if player.magic == MAGIC_FIRE
					local fireball = P_SPMAngle(player.mo, MT_FIREBALL, player.mo.angle)
					if fireball
						fireball.color = firecolors[P_RandomRange(1, 6)]
						fireball.colorized = true
						fireball.flags = $|MF_NOGRAVITY
						P_InstaThrust(fireball, player.mo.angle, 20*FRACUNIT)
						P_SetObjectMomZ(fireball, 0)
						S_StartSound(player.mo, sfx_mario7)
					end	
					
					local gotox = FixedMul(50*FRACUNIT, cos(player.mo.angle))
					local gotoy = FixedMul(50*FRACUNIT, sin(player.mo.angle))
					
					SpawnMagicPentogram(player.mo, firecolors[P_RandomRange(1, 6)], gotox, gotoy, player.mo.height/2, false, 15, player.magicrollangle)
				end
				if player.magic == MAGIC_ICE
					local nextx = FixedMul(150*FRACUNIT, cos(player.mo.angle))
					local nexty = FixedMul(150*FRACUNIT, sin(player.mo.angle))
				
					local icecube = P_SpawnMobjFromMobj(player.mo, 0, 0, -130*FRACUNIT, MT_ICECUBE)
					if icecube
						icecube.fuse = 100
						icecube.angle = player.mo.angle
						icecube.shatter = 100
						icecube.tracer = player.mo
						S_StartSound(icecube, sfx_iceb)
					end	
					
					local gotox = FixedMul(50*FRACUNIT, cos(player.mo.angle))
					local gotoy = FixedMul(50*FRACUNIT, sin(player.mo.angle))
					
					SpawnMagicPentogram(player.mo, icecolors[P_RandomRange(1, 6)], gotox, gotoy, player.mo.height/2, false, 15, player.magicrollangle)
				end
				if player.magic == MAGIC_WATER
					local bubble = P_SPMAngle(player.mo, MT_THOK, player.mo.angle)
					if bubble
						bubble.fuse = 250
						bubble.state = S_EXTRALARGEBUBBLE
						bubble.flags = MF_NOGRAVITY
						bubble.bubble = true
						bubble.gotenemy = nil
						bubble.tracer = player.mo
						bubble.scale = FRACUNIT + FRACUNIT/2
						P_InstaThrust(bubble, player.mo.angle, 20*FRACUNIT)
						P_SetObjectMomZ(bubble, 0)
						S_StartSound(player.mo, sfx_prloop)
					end	
					
					local gotox = FixedMul(50*FRACUNIT, cos(player.mo.angle))
					local gotoy = FixedMul(50*FRACUNIT, sin(player.mo.angle))
					
					SpawnMagicPentogram(player.mo, watercolors[P_RandomRange(1, 6)], gotox, gotoy, player.mo.height/2, false, 15, player.magicrollangle)
				end
				if player.magic == MAGIC_WIND
					local ghost = P_SpawnGhostMobj(player.mo)
					ghost.destscale = FRACUNIT + FRACUNIT/5
					ghost.color = SKINCOLOR_AETHER
					ghost.colorized = true
					ghost.fuse = 2
					S_StartSound(player.mo, sfx_wdjump)
					SpawnMagicPentogram(player.mo, windcolors[P_RandomRange(1, 6)], 0, 0, FRACUNIT/15, true, 15, player.magicrollangle)
				end
				if player.magic == MAGIC_BOMB
					local fangbomb = P_SPMAngle(player.mo, MT_FBOMB, player.mo.angle)
					if fangbomb
						fangbomb.tracer = player.mo
						fangbomb.color = bombcolors[P_RandomRange(1, 6)]
						fangbomb.colorized = true
						P_SetObjectMomZ(fangbomb, 5*FRACUNIT)
						S_StartSound(player.mo, sfx_peww)
					end	
					local gotox = FixedMul(50*FRACUNIT, cos(player.mo.angle))
					local gotoy = FixedMul(50*FRACUNIT, sin(player.mo.angle))
					
					SpawnMagicPentogram(player.mo, bombcolors[P_RandomRange(1, 6)], gotox, gotoy, player.mo.height/2, false, 15, player.magicrollangle)
				end
				if player.magic == MAGIC_MAGNET
					if player.magnet
						if target
							local angle = R_PointToAngle2(player.magnet.x, player.magnet.y, target.x, target.y)
							local dist = R_PointToDist2(player.magnet.x, player.magnet.y, target.x, target.y)
							P_InstaThrust(target, angle+ANGLE_180, 30*FRACUNIT)
							S_StartSound(player.mo, sfx_shield)
						else
							local elec = P_SPMAngle(player.mo, MT_JETTBULLET, player.mo.angle)
							elec.color = SKINCOLOR_YELLOW
							elec.colorized = true
							elec.tracer = player.mo
							P_TeleportMove(elec, player.magnet.x, player.magnet.y, player.magnet.z)
							S_StartSound(player.mo, sfx_turhit)
						end
						SpawnMagicPentogram(player.mo, magnetcolors[P_RandomRange(1, 6)], player.magnet.x, player.magnet.y, player.mo.height/2, false, 15, player.magicrollangle)
					end	
				end
				if player.magic == MAGIC_ELEC
					S_StartSound(player.mo, sfx_s3k42)
					local instashield = P_SPMAngle(player.mo, MT_JETTBULLET, player.mo.angle)
					instashield.momx = player.mo.momx
					instashield.momy = player.mo.momy
					instashield.momz = player.mo.momz
					instashield.tracer = player.mo
					instashield.colorized = true
					instashield.fuse = 14
					instashield.color = SKINCOLOR_WHITE
					instashield.scale = FRACUNIT*50
					instashield.state = S_INSTASHIELD1
					instashield.spriteyscale = FRACUNIT/42
					instashield.spritexscale = FRACUNIT/42
				end
			end
			if player.spin
			and player.specialuses
			and player.magic == MAGIC_WIND
				local ghost = P_SpawnGhostMobj(player.mo)
				ghost.destscale = FRACUNIT - FRACUNIT/5
				ghost.color = SKINCOLOR_AETHER
				ghost.colorized = true
				ghost.fuse = 2
				player.mo.state = S_PLAY_CLIMB
				player.drawangle = (30*player.realtime)*ANG1
				player.pflags = $|PF_SPINNING
					
				for i = 1, 4
					local ns = player.drawangle + (ANGLE_90*i)
					
					local windx = FixedMul(50*FRACUNIT, sin(ns))
					local windy = FixedMul(50*FRACUNIT, cos(ns))
						
					local wind = P_SpawnMobjFromMobj(player.mo, windx, windy, player.mo.height/2, MT_DUSTLAYER)
					wind.frame = $|FF_PAPERSPRITE
					wind.fuse = 5
					wind.scale = FRACUNIT/2
					wind.angle = R_PointToAngle2(wind.x, wind.y, player.mo.x+player.mo.momx, player.mo.y+player.mo.momy)
					wind.colorized = true
					wind.color = windcolors[P_RandomRange(1, 6)]
				end	
			end
			player.spin = true
		else
			if player.spin == true
			and player.magic == MAGIC_WIND
			and player.pflags & PF_SPINNING
				player.mo.state = S_PLAY_STND
				player.drawangle = player.mo.angle
				player.pflags = $ & ~PF_SPINNING
				local ghost = P_SpawnGhostMobj(player.mo)
				ghost.destscale = FRACUNIT + FRACUNIT/5
				ghost.color = SKINCOLOR_AETHER
				ghost.colorized = true
				ghost.fuse = 2
			end
			player.spin = false
		end
		if player.cmd.buttons & BT_JUMP
			player.jump = true
		else
			player.jump = false
		end
	end
end)

local function BubbleCatch(mo, bubble)
    if mo.valid and bubble and bubble.valid and (bubble.type == MT_THOK) and (bubble.state == S_EXTRALARGEBUBBLE)
	and (mo.flags & MF_ENEMY or mo.flags & MF_MONITOR)
	and not bubble.gotenemy and not mo.catched
	and bubble.bubble
    
        if (mo.z > bubble.z+bubble.height)
        or (bubble.z > mo.z+mo.height)
        return
        end

        bubble.gotenemy = mo
        return true

    end
end

addHook("MobjCollide", BubbleCatch, MT_NULL)
addHook("MobjMoveCollide", BubbleCatch, MT_NULL)

addHook("MobjThinker", function(mo)
	if mo.state == S_EXTRALARGEBUBBLE
	and mo.bubble
	and mo.type == MT_THOK
		if mo.gotenemy and mo.gotenemy.valid
			mo.gotenemy.momx = mo.momx
			mo.gotenemy.momy = mo.momy
			mo.gotenemy.momz = mo.momz
			mo.gotenemy.catched = true
			
			P_TeleportMove(mo.gotenemy, mo.x, mo.y, mo.z+2*FRACUNIT)
			
			if mo.fuse <= 1
				P_SetObjectMomZ(mo.gotenemy, 5*FRACUNIT)
				P_SpawnMobjFromMobj(mo, 0, 0, 0, MT_UWEXPLODE)
				if mo.gotenemy and mo.gotenemy.valid
					mo.gotenemy.catched = false
					mo.gotenemy = nil
				end	
			end
			
			local dist = R_PointToDist2(mo.x, mo.y, mo.tracer.x, mo.tracer.y)
			local angle = R_PointToAngle2(mo.x, mo.y, mo.tracer.x, mo.tracer.y)
			if dist > 150*FRACUNIT
				P_Thrust(mo, angle, FRACUNIT)
				mo.fuse = 50
			end
			if mo.z < (mo.tracer.z - 50*FRACUNIT)
				P_SetObjectMomZ(mo, FRACUNIT, true)
			end
			if mo.z > (mo.tracer.z+mo.height + 50*FRACUNIT)
				P_SetObjectMomZ(mo, -FRACUNIT, true)
			end
		end
	end
end)

addHook("ShouldDamage", function(mobj, mo, source, damage, damagetype)
	if mobj.type == MT_RING_BOX and mo.type == MT_PLAYER
	and mo.player
		if mo.player.specialuses < 10
			mo.player.specialuses = $ + 2
		end
	end
	if mo and mo.tracer
	and mobj and mo.tracer != mobj
		P_DamageMobj(mobj, mo.tracer, mo.tracer)
	end
	if mobj.type == MT_PLAYER
	and damagetype == DMG_FIRE
		if mobj.player.specialuses
		and mobj.player.magic == MAGIC_FIRE
		and not (mobj.player.powers[pw_shield] & SH_PROTECTFIRE)
			if (leveltime%50) == 0
				mobj.player.specialuses = $ - 1
			end
			return false
		end
	end
	if mobj.type == MT_PLAYER
	and mo and mo.tracer == mobj
		return false
	end
	if mobj.type == MT_PLAYER
	and mo and mo.catched
		return false
	end
end)

addHook("AbilitySpecial", function(player)
	if player.valid and player.playerstate == PST_LIVE
	and player.mo.skin == "magicmetal" and not (player.powers[pw_carry] == CR_NIGHTSMODE)
		if not (player.pflags & PF_THOKKED)
		and player.specialuses
			if player.magic == MAGIC_NONE
				player.metaljump = 25
				P_SetObjectMomZ(player.mo, -5*FRACUNIT, true)
				local ghost = P_SpawnGhostMobj(player.mo)
				ghost.destscale = player.mo.scale + FRACUNIT/2
				ghost.colorized = true
				ghost.color = nonecolors[P_RandomRange(1, 5)]
				S_StartSound(player.mo, sfx_mswarp)
				player.mo.state = S_PLAY_CLIMB
				player.pflags = $|PF_JUMPED|PF_NOJUMPDAMAGE
			end
			if player.magic == MAGIC_FIRE
			and not player.speeny
				player.speeny = 50
				local ghost = P_SpawnGhostMobj(player.mo)
				ghost.destscale = player.mo.scale + FRACUNIT/2
				ghost.colorized = true
				ghost.color = firecolors[P_RandomRange(1, 6)]
				P_SetObjectMomZ(player.mo, 5*FRACUNIT)
				P_InstaThrust(player.mo, player.mo.angle, 40*FRACUNIT)
				S_StartSound(player.mo, sfx_s3k3e)
			end
			if player.magic == MAGIC_ICE
			and not player.speeny
				//player.iceroad = 25
				player.stap = 25
				local ghost = P_SpawnGhostMobj(player.mo)
				ghost.destscale = player.mo.scale + FRACUNIT/2
				ghost.colorized = true
				ghost.color = firecolors[P_RandomRange(1, 6)]
				S_StartSound(player.mo, sfx_shield)
				//player.jump = true
			end
			if player.magic == MAGIC_WATER
				player.bubblebounce = 50
				local ghost = P_SpawnGhostMobj(player.mo)
				ghost.destscale = player.mo.scale + FRACUNIT/2
				ghost.colorized = true
				ghost.color = watercolors[P_RandomRange(1, 6)]
				P_SetObjectMomZ(player.mo, -15*FRACUNIT, true)
			end
			if player.magic == MAGIC_WIND
			and not player.flying
				local ghost = P_SpawnGhostMobj(player.mo)
				ghost.destscale = player.mo.scale + FRACUNIT/2
				ghost.colorized = true
				ghost.color = windcolors[P_RandomRange(1, 6)]
				//player.flying = 25
				P_SetObjectMomZ(player.mo, 16*FRACUNIT)
				S_StartSound(player.mo, sfx_wdjump)
				P_Telekinesis(player, 30*FRACUNIT, 500*FRACUNIT)
				for i = 1, 15
					local ns = i*(72*ANG1)
					local dust = P_SpawnMobjFromMobj(player.mo, 0, 0, 0, MT_DUST)
					dust.fuse = 15
					dust.scale = FRACUNIT*2
					//dust.away = true
					P_SetObjectMomZ(dust, -15*FRACUNIT)
				end
			end
			if player.magic == MAGIC_BOMB
			//and player.mo.momz*P_MobjFlip(player.mo) < -5*FRACUNIT
				local bomb = P_SpawnMobjFromMobj(player.mo, 0, 0, -15*FRACUNIT, MT_FBOMB)
				bomb.colorized = true
				bomb.color = bombcolors[P_RandomRange(1, 6)]
				bomb.tracer = player.mo
				P_SetObjectMomZ(bomb, -abs(player.mo.momz) - 10*FRACUNIT)
				P_SetObjectMomZ(player.mo, abs(player.mo.momz))
				S_StartSound(player.mo, sfx_ngjump)
				//player.specialuses = $ - 1
			end
			if player.magic == MAGIC_MAGNET
			and not player.float
				player.float = 35*3
				S_StartSound(player.mo, sfx_attrsg)
				local ghost = P_SpawnGhostMobj(player.mo)
				ghost.destscale = player.mo.scale + FRACUNIT/2
				ghost.colorized = true
				ghost.color = magnetcolors[P_RandomRange(1, 6)]
			end
			if player.magic == MAGIC_ELEC
				player.zoomers = 5
				P_InstaThrust(player.mo, player.drawangle, 60*FRACUNIT)
				S_StartSound(player.mo, sfx_spdpad)
				local ghost = P_SpawnGhostMobj(player.mo)
				ghost.destscale = player.mo.scale + FRACUNIT/2
				ghost.colorized = true
				ghost.color = eleccolors[P_RandomRange(1, 6)]
				
				local gotox = FixedMul(50*FRACUNIT, cos(player.mo.angle))
				local gotoy = FixedMul(50*FRACUNIT, sin(player.mo.angle))
					
				SpawnMagicPentogram(player.mo, eleccolors[P_RandomRange(1, 6)], gotox, gotoy, player.mo.height/2, false, 15, player.magicrollangle)
				
				/*local randomx = FixedMul(P_RandomRange(-150, 150)*FRACUNIT, sin(P_RandomRange(0, 360)*ANG1))
				local randomy = FixedMul(P_RandomRange(-150, 150)*FRACUNIT, cos(P_RandomRange(0, 360)*ANG1))
				player.target = P_LookForEnemies(player, false, false)
				
				local thun
				if player.target
					thun = P_SpawnMobjFromMobj(player.target, 0, 0, 0, MT_THOK)
					//P_TeleportMove(thun, player.target.x, player.target.y, player.target.z)
					P_DamageMobj(player.target, player.mo, player.mo)
				else
					P_SetObjectMomZ(player.mo, 0)
					P_InstaThrust(player.mo, player.mo.angle, 60*FRACUNIT)
					thun = P_SpawnMobjFromMobj(player.mo, randomx, randomy, player.mo.z, MT_THOK)
				end
				if thun
					thun.state = S_THUNDER1
					thun.tracer = player.mo
					S_StartSound(thun, sfx_rail2)
				end*/
			end
			if player.magic != MAGIC_BOMB
				player.pflags = $|PF_THOKKED
			end	
			if player.magic != MAGIC_NONE
			//or not (player.mo.momz*P_MobjFlip(player.mo) < -5*FRACUNIT and player.magic == MAGIC_BOMB)
				player.specialuses = $ - 1
			end	
		end
	end
end)

addHook("TouchSpecial", function(mobj, mo)
	if mo and mo.player and mobj
		if mo.skin == "magicmetal"
			if mo.player.halfuse < 5
				mo.player.halfuse = $ + 1
			else
				if mo.player.specialuses < 10
					mo.player.specialuses = $ + 1
					mo.player.halfuse = 0
				end	
			end
		end
	end
end, MT_RING)

local function HUD(v, player, camera)
	if player.valid and player.playerstate == PST_LIVE
	and player.mo.skin == "magicmetal" and not (player.powers[pw_carry] == CR_NIGHTSMODE)
		v.drawScaled(15*FRACUNIT, 140*FRACUNIT, FRACUNIT/7, v.cachePatch("MAGCHS"), V_HUDTRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_20TRANS, v.getColormap("sonic", player.mo.color))
		v.drawScaled(45*FRACUNIT, 110*FRACUNIT, FRACUNIT/4, v.cachePatch("MAGICUSES"), V_HUDTRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_20TRANS, v.getColormap("sonic", player.mo.color))
		if player.specialuses
			v.drawString(57, 115, player.specialuses, V_HUDTRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_20TRANS, "center")
		else
			v.drawString(57, 115, "0", V_HUDTRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_20TRANS, "center")
		end	
		if player.magic == MAGIC_NONE
			v.drawScaled(15*FRACUNIT, 110*FRACUNIT, FRACUNIT/4, v.cachePatch("MAGICPANELX"), V_HUDTRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_20TRANS, v.getColormap("sonic", player.mo.color))
		end
		if player.magic == MAGIC_FIRE
			v.drawScaled(15*FRACUNIT, 110*FRACUNIT, FRACUNIT/4, v.cachePatch("MAGICPANEL1"), V_HUDTRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_20TRANS, v.getColormap("sonic", player.mo.color))
		end
		if player.magic == MAGIC_ICE
			v.drawScaled(15*FRACUNIT, 110*FRACUNIT, FRACUNIT/4, v.cachePatch("MAGICPANEL2"), V_HUDTRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_20TRANS, v.getColormap("sonic", player.mo.color))
		end
		if player.magic == MAGIC_WATER
			v.drawScaled(15*FRACUNIT, 110*FRACUNIT, FRACUNIT/4, v.cachePatch("MAGICPANEL3"), V_HUDTRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_20TRANS, v.getColormap("sonic", player.mo.color))
		end
		if player.magic == MAGIC_WIND
			v.drawScaled(15*FRACUNIT, 110*FRACUNIT, FRACUNIT/4, v.cachePatch("MAGICPANEL4"), V_HUDTRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_20TRANS, v.getColormap("sonic", player.mo.color))
		end
		if player.magic == MAGIC_BOMB
			v.drawScaled(15*FRACUNIT, 110*FRACUNIT, FRACUNIT/4, v.cachePatch("MAGICPANEL5"), V_HUDTRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_20TRANS, v.getColormap("sonic", player.mo.color))
		end
		if player.magic == MAGIC_MAGNET
			v.drawScaled(15*FRACUNIT, 110*FRACUNIT, FRACUNIT/4, v.cachePatch("MAGICPANEL6"), V_HUDTRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_20TRANS, v.getColormap("sonic", player.mo.color))
		end
		if player.magic == MAGIC_ELEC
			v.drawScaled(15*FRACUNIT, 110*FRACUNIT, FRACUNIT/4, v.cachePatch("MAGICPANEL7"), V_HUDTRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_20TRANS, v.getColormap("sonic", player.mo.color))
		end
	end
end
hud.add(HUD, "game")